import { Injectable, defineInjectable, inject } from '@angular/core';
import { HttpParams, HttpHeaders, HttpClient } from '@angular/common/http';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?=} transformers
 * @return {?}
 */
function reduceRequest(transformers = []) {
    return (requestData) => {
        return transformers.reduce((transformedRequest, transformer) => {
            transformedRequest = transformer(transformedRequest);
            return transformedRequest;
        }, requestData);
    };
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?} params
 * @return {?}
 */
function normalizeParams(params) {
    if (!params) {
        return new HttpParams();
    }
    if (params instanceof HttpParams) {
        return params;
    }
    return Object.entries(params)
        .reduce((httpParams, entry) => {
        const [key, param] = entry;
        return httpParams.append(key, param.toString());
    }, new HttpParams());
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?=} options
 * @return {?}
 */
function normalizeOptions(options = {}) {
    return Object.assign({}, options, { params: normalizeParams(options.params), headers: new HttpHeaders(options.headers) });
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?} transformers
 * @return {?}
 */
function transformRequest(transformers) {
    /** @type {?} */
    const reduceRequestTransformers = reduceRequest(transformers);
    return (request) => {
        /** @type {?} */
        const normalizedRequest = {
            body: request.body,
            options: normalizeOptions(request.options),
        };
        return reduceRequestTransformers(normalizedRequest);
    };
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class Api {
    /**
     * @param {?} http
     * @param {?} config
     */
    constructor(http, config) {
        this.http = http;
        this.config = config;
        this.transformRequest = (request) => request;
        this.http = http;
        this.config = config;
        this.baseUrl = this.config.urls[this.config.environment];
        this.transformRequest = transformRequest(this.config.transformers);
    }
    /**
     * @private
     * @param {?} path
     * @return {?}
     */
    getUrl(path) {
        return `${this.baseUrl}${path}`;
    }
    /**
     * @param {?} path
     * @param {?} options
     * @return {?}
     */
    get(path, options) {
        /** @type {?} */
        const url = this.getUrl(path);
        /** @type {?} */
        const preparedRequest = this.transformRequest({ options });
        return this.http.get(url, preparedRequest.options);
    }
    /**
     * @param {?} path
     * @param {?} body
     * @param {?} options
     * @return {?}
     */
    post(path, body, options) {
        /** @type {?} */
        const url = this.getUrl(path);
        /** @type {?} */
        const preparedRequest = this.transformRequest({ body, options });
        return this.http.post(url, preparedRequest.body, preparedRequest.options);
    }
    /**
     * @param {?} path
     * @param {?} body
     * @param {?} options
     * @return {?}
     */
    put(path, body, options) {
        /** @type {?} */
        const url = this.getUrl(path);
        /** @type {?} */
        const preparedRequest = this.transformRequest({ body, options });
        return this.http.put(url, preparedRequest.body, preparedRequest.options);
    }
    /**
     * @param {?} path
     * @param {?} options
     * @return {?}
     */
    delete(path, options) {
        /** @type {?} */
        const url = this.getUrl(path);
        /** @type {?} */
        const preparedRequest = this.transformRequest({ options });
        return this.http.delete(url, preparedRequest.options);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ApiBuilderService {
    /**
     * @param {?} http
     */
    constructor(http) {
        this.http = http;
    }
    /**
     * @param {?} config
     * @return {?}
     */
    build(config) {
        return new Api(this.http, config);
    }
}
ApiBuilderService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */
ApiBuilderService.ctorParameters = () => [
    { type: HttpClient }
];
/** @nocollapse */ ApiBuilderService.ngInjectableDef = defineInjectable({ factory: function ApiBuilderService_Factory() { return new ApiBuilderService(inject(HttpClient)); }, token: ApiBuilderService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { ApiBuilderService, Api };

//# sourceMappingURL=ng-rapi.js.map