import { Injectable, defineInjectable, inject } from '@angular/core';
import { __assign, __read } from 'tslib';
import { HttpParams, HttpHeaders, HttpClient } from '@angular/common/http';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?=} transformers
 * @return {?}
 */
function reduceRequest(transformers) {
    if (transformers === void 0) { transformers = []; }
    return function (requestData) {
        return transformers.reduce(function (transformedRequest, transformer) {
            transformedRequest = transformer(transformedRequest);
            return transformedRequest;
        }, requestData);
    };
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?} params
 * @return {?}
 */
function normalizeParams(params) {
    if (!params) {
        return new HttpParams();
    }
    if (params instanceof HttpParams) {
        return params;
    }
    return Object.entries(params)
        .reduce(function (httpParams, entry) {
        var _a = __read(entry, 2), key = _a[0], param = _a[1];
        return httpParams.append(key, param.toString());
    }, new HttpParams());
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?=} options
 * @return {?}
 */
function normalizeOptions(options) {
    if (options === void 0) { options = {}; }
    return __assign({}, options, { params: normalizeParams(options.params), headers: new HttpHeaders(options.headers) });
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?} transformers
 * @return {?}
 */
function transformRequest(transformers) {
    /** @type {?} */
    var reduceRequestTransformers = reduceRequest(transformers);
    return function (request) {
        /** @type {?} */
        var normalizedRequest = {
            body: request.body,
            options: normalizeOptions(request.options),
        };
        return reduceRequestTransformers(normalizedRequest);
    };
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var Api = /** @class */ (function () {
    function Api(http, config) {
        this.http = http;
        this.config = config;
        this.transformRequest = function (request) { return request; };
        this.http = http;
        this.config = config;
        this.baseUrl = this.config.urls[this.config.environment];
        this.transformRequest = transformRequest(this.config.transformers);
    }
    /**
     * @private
     * @param {?} path
     * @return {?}
     */
    Api.prototype.getUrl = /**
     * @private
     * @param {?} path
     * @return {?}
     */
    function (path) {
        return "" + this.baseUrl + path;
    };
    /**
     * @param {?} path
     * @param {?} options
     * @return {?}
     */
    Api.prototype.get = /**
     * @param {?} path
     * @param {?} options
     * @return {?}
     */
    function (path, options) {
        /** @type {?} */
        var url = this.getUrl(path);
        /** @type {?} */
        var preparedRequest = this.transformRequest({ options: options });
        return this.http.get(url, preparedRequest.options);
    };
    /**
     * @param {?} path
     * @param {?} body
     * @param {?} options
     * @return {?}
     */
    Api.prototype.post = /**
     * @param {?} path
     * @param {?} body
     * @param {?} options
     * @return {?}
     */
    function (path, body, options) {
        /** @type {?} */
        var url = this.getUrl(path);
        /** @type {?} */
        var preparedRequest = this.transformRequest({ body: body, options: options });
        return this.http.post(url, preparedRequest.body, preparedRequest.options);
    };
    /**
     * @param {?} path
     * @param {?} body
     * @param {?} options
     * @return {?}
     */
    Api.prototype.put = /**
     * @param {?} path
     * @param {?} body
     * @param {?} options
     * @return {?}
     */
    function (path, body, options) {
        /** @type {?} */
        var url = this.getUrl(path);
        /** @type {?} */
        var preparedRequest = this.transformRequest({ body: body, options: options });
        return this.http.put(url, preparedRequest.body, preparedRequest.options);
    };
    /**
     * @param {?} path
     * @param {?} options
     * @return {?}
     */
    Api.prototype.delete = /**
     * @param {?} path
     * @param {?} options
     * @return {?}
     */
    function (path, options) {
        /** @type {?} */
        var url = this.getUrl(path);
        /** @type {?} */
        var preparedRequest = this.transformRequest({ options: options });
        return this.http.delete(url, preparedRequest.options);
    };
    return Api;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ApiBuilderService = /** @class */ (function () {
    function ApiBuilderService(http) {
        this.http = http;
    }
    /**
     * @param {?} config
     * @return {?}
     */
    ApiBuilderService.prototype.build = /**
     * @param {?} config
     * @return {?}
     */
    function (config) {
        return new Api(this.http, config);
    };
    ApiBuilderService.decorators = [
        { type: Injectable, args: [{
                    providedIn: 'root'
                },] }
    ];
    /** @nocollapse */
    ApiBuilderService.ctorParameters = function () { return [
        { type: HttpClient }
    ]; };
    /** @nocollapse */ ApiBuilderService.ngInjectableDef = defineInjectable({ factory: function ApiBuilderService_Factory() { return new ApiBuilderService(inject(HttpClient)); }, token: ApiBuilderService, providedIn: "root" });
    return ApiBuilderService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { ApiBuilderService, Api };

//# sourceMappingURL=ng-rapi.js.map